@@ -1,13 +1,37 @@
 import math
 import string
+import sys
 from datetime import datetime, timedelta
+
+try:
+    from functools import lru_cache
+except ImportError:
+    from functools32 import lru_cache
+
 from itertools import cycle
 
 import numpy as np
 import pandas as pd
+import pytz
 
 from .utils import find_package_file
 
+COLUMN_TYPES = [
+    "bool",
+    "int",
+    "float",
+    "str",
+    "categories",
+    "boolean",
+    "Int64",
+    "date",
+    "datetime",
+    "timedelta",
+]
+
+if sys.version_info < (3,):
+    COLUMN_TYPES = [type for type in COLUMN_TYPES if type != "boolean"]
+
 
 def get_countries():
     """A Pandas DataFrame with the world countries (from the world bank data)"""
@@ -54,7 +78,7 @@ def get_dict_of_test_dfs(N=100, M=100):
                 [None, False, True, False],
             ],
             columns=list("abcd"),
-            dtype="boolean",
+            dtype="boolean" if sys.version_info > (3,) else "bool",
         ),
         "int": pd.DataFrame(
             [[-1, 2, -3, 4, -5], [6, -7, 8, -9, 10]], columns=list("abcde")
@@ -80,8 +104,26 @@ def get_dict_of_test_dfs(N=100, M=100):
         ),
         "time": pd.DataFrame(
             {
-                "datetime": [datetime(2000, 1, 1), datetime(2001, 1, 1)],
-                "timedelta": [timedelta(days=2), timedelta(seconds=50)],
+                "datetime": [datetime(2000, 1, 1), datetime(2001, 1, 1), pd.NaT],
+                "timestamp": [
+                    pd.NaT,
+                    datetime(2000, 1, 1, 18, 55, 33),
+                    datetime(
+                        2001,
+                        1,
+                        1,
+                        18,
+                        55,
+                        55,
+                        456654,
+                        tzinfo=pytz.timezone("US/Eastern"),
+                    ),
+                ],
+                "timedelta": [
+                    timedelta(days=2),
+                    timedelta(seconds=50),
+                    pd.NaT - datetime(2000, 1, 1),
+                ],
             }
         ),
         "date_range": pd.DataFrame(
@@ -111,8 +153,8 @@ def get_dict_of_test_dfs(N=100, M=100):
         ),
         "wide": pd.DataFrame(
             NM_values,
-            index=[f"row_{i}" for i in range(N)],
-            columns=[f"column_{j}" for j in range(M)],
+            index=["row_{}".format(i) for i in range(N)],
+            columns=["column_{}".format(j) for j in range(M)],
         ),
         "long_column_names": pd.DataFrame(
             {
@@ -145,5 +187,67 @@ def get_dict_of_test_series():
         if len(df.columns) > 6:
             continue
         for col in df.columns:
-            series[f"{df_name}.{col}"] = df[col]
+            series["{}.{}".format(df_name, col)] = df[col]
     return series
+
+
+@lru_cache()
+def generate_date_series():
+    return pd.Series(pd.date_range("1677-09-23", "2262-04-10", freq="D"))
+
+
+def generate_random_series(rows, type):
+    if type == "bool":
+        return pd.Series(np.random.binomial(n=1, p=0.5, size=rows), dtype=bool)
+    if type == "boolean":
+        x = generate_random_series(rows, "bool").astype(type)
+        x.loc[np.random.binomial(n=1, p=0.1, size=rows) == 0] = pd.NA
+        return x
+    if type == "int":
+        return pd.Series(np.random.geometric(p=0.1, size=rows), dtype=int)
+    if type == "Int64":
+        x = generate_random_series(rows, "int").astype(type)
+        if sys.version_info >= (3,):
+            x.loc[np.random.binomial(n=1, p=0.1, size=rows) == 0] = pd.NA
+        return x
+    if type == "float":
+        x = pd.Series(np.random.normal(size=rows), dtype=float)
+        x.loc[np.random.binomial(n=1, p=0.05, size=rows) == 0] = float("nan")
+        x.loc[np.random.binomial(n=1, p=0.05, size=rows) == 0] = float("inf")
+        x.loc[np.random.binomial(n=1, p=0.05, size=rows) == 0] = float("-inf")
+        return x
+    if type == "str":
+        return get_countries()["name"].sample(n=rows, replace=True)
+    if type == "categories":
+        x = generate_random_series(rows, "str")
+        return pd.Series(x, dtype="category")
+    if type == "date":
+        x = generate_date_series().sample(rows, replace=True)
+        x.loc[np.random.binomial(n=1, p=0.1, size=rows) == 0] = pd.NaT
+        return x
+    if type == "datetime":
+        x = generate_random_series(rows, "date") + np.random.uniform(
+            0, 1, rows
+        ) * pd.Timedelta(1, unit="D")
+        return x
+    if type == "timedelta":
+        x = generate_random_series(rows, "datetime").sample(frac=1)
+        return x.diff()
+    raise NotImplementedError(type)
+
+
+def generate_random_df(rows, columns, column_types=COLUMN_TYPES):
+    rows = int(rows)
+    types = np.random.choice(column_types, size=columns)
+    columns = [
+        "Column{}OfType{}".format(col, type.title()) for col, type in enumerate(types)
+    ]
+
+    series = {
+        col: generate_random_series(rows, type) for col, type in zip(columns, types)
+    }
+    index = pd.Index(range(rows))
+    for x in series.values():
+        x.index = index
+
+    return pd.DataFrame(series)
